--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.fpoinc_pkg.all;

--apply 50 MHz to clk/ena

package demo_pkg is

  component demo is
    port (clk, ena, reset : in std_ulogic; ps2c, ps2d : inout std_ulogic; vc : out vgaout);
  end component;

end demo_pkg;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.fpoinc_pkg.all;
use work.fpo_pkg.all;

entity demo is
  port (clk, ena, reset : in std_ulogic; ps2c, ps2d : inout std_ulogic; vc : out vgaout);
end demo;

architecture arch_demo of demo is

  signal acqena, ps2ena, demoena : std_ulogic;
  signal ps2divider              : natural range 0 to 255;
  signal acqdivider              : natural range 0 to 2;
  signal demo_divider            : natural range 0 to 10;
  signal demo_counter            : unsigned(7 downto 0);
  signal fpobus                  : fpo_data_typ;
  signal trg_input               : std_ulogic;

begin

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = ps2ena then
          ps2divider <= 0;
        else
          ps2divider <= ps2divider + 1;
        end if;
      end if;
    end if;
  end process;

  ps2ena <= '1' when 255 <= ps2divider else '0';

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = acqena then
          acqdivider <= 0;
        else
          acqdivider <= acqdivider + 7;
--          acqdivider <= acqdivider + 1;
        end if;
      end if;
    end if;
  end process;

  acqena <= '1' when 2 <= acqdivider else '0';

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = demoena then
          demo_divider <= 0;
        else
          demo_divider <= demo_divider + 1;
        end if;
      end if;
    end if;
  end process;

  demoena <= '1' when 10 <= demo_divider else '0';

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = demoena then
          demo_counter <= demo_counter + 1;
        end if;
      end if;
    end if;
  end process;

  fpobus(0) <= not demo_counter(7);

  fpobus(8 downto 1) <= std_ulogic_vector(demo_counter);

  fpobus(16 downto 9) <= std_ulogic_vector(demo_counter);

  fpobus(24 downto 17) <= std_ulogic_vector(demo_counter);

  fpobus(32 downto 25) <= std_ulogic_vector(demo_counter xor ('0' & demo_counter(7 downto 1)));  --gray code

  fpobus(38 downto 33) <= std_ulogic_vector(demo_counter(5 downto 0) xor ('0' & demo_counter(5 downto 1)));  --gray code

  fpobus(39) <= ps2c;

  fpobus(40) <= ps2d;

  trg_input <= not demo_counter(7);

  fd : fpo port map(
    clk => clk,
    ena => acqena,

    vclk => clk,
    vena => ena, --'1',

    ps2clk => clk,
    ps2ena => ps2ena,
    reset  => reset,

    trg_input => trg_input,
    d         => fpobus,
    ps2c      => ps2c,
    ps2d      => ps2d,
    vc        => vc);

end arch_demo;
