--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.videomode_pkg.all;
package axissync_pkg is

  component axissync is
    generic(frontporch : positive;
            syncpulse  : positive;
            backporch  : positive;
            visible    : positive);
    port (clk, ena : in std_ulogic; s, v, e : out std_ulogic; coord : out std_ulogic_vector(wh(visible) downto 0));
  end component;

end axissync_pkg;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.videomode_pkg.all;

entity axissync is
  generic(frontporch : natural;
          syncpulse  : natural;
          backporch  : natural;
          visible    : positive);
  port (clk, ena : in std_ulogic; s, v, e : out std_ulogic; coord : out std_ulogic_vector(wh(visible) downto 0));
end axissync;

architecture arch_axissync of axissync is

  signal fpc : natural range 0 to frontporch;
  signal spc : natural range 0 to syncpulse;
  signal bpc : natural range 0 to backporch;
  signal vc  : natural range 0 to visible;
  signal fpcp, spcp, bpcp, vcp, vcr : std_ulogic;

begin

  fpcp <= '1' when (frontporch) = fpc else '0';
  spcp <= '1' when (syncpulse) = spc  else '0';
  bpcp <= '1' when (backporch) = bpc  else '0';
  vcp  <= '1' when (visible - 1) = vc else '0';
  vcr  <= fpcp and spcp and bpcp and vcp;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = vcr then
          fpc <= 0;
        else
          if '0' = fpcp then
            fpc <= fpc + 1;
          end if;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = vcr then
          spc <= 0;
        else
          if '1' = (fpcp and not spcp) then
            spc <= spc + 1;
          end if;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = vcr then
          bpc <= 0;
        else
          if '1' = (spcp and fpcp and not bpcp) then
            bpc <= bpc + 1;
          end if;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = vcr then
          vc <= 0;
        else
--    if '1' = (bpcp and spcp and fpcp and not vcp) then
          if '1' = (bpcp and spcp and fpcp) then
            vc <= vc + 1;
          end if;
        end if;
      end if;
    end if;
  end process;

  coord <= std_ulogic_vector(to_unsigned(vc, wh(visible) + 1));

  s <= fpcp and not spcp;

  v <= bpcp and spcp and fpcp;

  e <= vcp and ena;

end arch_axissync;
