--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.fpoinc_pkg.all;
use work.videomode_pkg.all;

package fpocore_pkg is

  component fpocore is
    port (clk, ena, vclk, vena, trg_input : in std_ulogic; d : fpo_data_typ; capture_ctrl : in capture_ctrl_rec; display_ctrl : in display_ctrl_rec; xcursor : in col_typ; vc : out vgaout);
  end component;

end fpocore_pkg;

library ieee;
use ieee.std_logic_1164.all;
use work.fpoinc_pkg.all;
use work.fpocfg_pkg.all;
use work.videomode_pkg.all;
use work.sweep_pkg.all;
use work.trinity_pkg.all;
use work.triram_pkg.all;
use work.trigger_pkg.all;
use work.trireg_pkg.all;
use work.graphics_pkg.all;

entity fpocore is
  port (clk, ena, vclk, vena, trg_input : in std_ulogic; d : fpo_data_typ; capture_ctrl : in capture_ctrl_rec; display_ctrl : in display_ctrl_rec; xcursor : in col_typ; vc : out vgaout);
end fpocore;

architecture arch_fpocore of fpocore is

  signal trg, sw, vactive, ready, fill, run, vpass, rena : std_ulogic;
  signal wadr, vadr, trgadr, sadr, badr                  : ramadr_typ;
  signal wsel, vsel                                      : std_ulogic_vector(2 downto 0);
  signal q                                               : fpo_data_typ;
  signal trgdrift, sdrift                                : drift_typ;

begin
--captute_ctrl syncing to clk ?

  triggger_ins : trigger port map(
    clk     => clk,
    ena     => ena,
    act     => capture_ctrl.act,
    d       => trg_input,
    trg_mod => capture_ctrl.mode,
    holdoff => 0,
    trg     => trg,
    ready   => ready);

  sweep_ins : sweep port map(
    clk          => clk,
    ena          => ena,
    trg          => trg,
    capture_ctrl => capture_ctrl,
    sw           => sw,
    fill         => fill,
    run          => run,
    adr          => wadr,
    trgadr       => trgadr,
    trgdrift     => trgdrift);

  trinity_ins : trinity port map(
    clk  => clk,
    ena  => ena,
    step => sw,
    hold => vactive,
    wsel => wsel,
    vsel => vsel);

  triram_ins : triram port map(
    clk  => clk,
    ena  => ena,
    rclk => vclk,
    rena => rena,                       --'1',
    wsel => wsel,
    vsel => vsel,
    wadr => wadr,
    radr => vadr,
    d    => d,
    q    => q);

  trireg_ins : trireg port map(
    clk    => clk,
    ena    => ena,
    wsel   => wsel,
    vsel   => vsel,
    idrift => trgdrift,
    iadr   => trgadr,
    odrift => sdrift,
    oadr   => sadr);

  badr <= (sadr - sdrift) mod ramlength;  --rem ramlength;

  grbaphics_ins : graphics port map(
    vclk         => vclk,
    vena         => vena,
    ready        => ready,
    fill         => fill,
    run          => run,
    badr         => badr,
    vadr         => vadr,
    d            => q,
    capture_ctrl => capture_ctrl,
    display_ctrl => display_ctrl,
    drift        => sdrift,
    xcursor      => xcursor,
    vc           => vc,
    vact         => vactive,
    vpass        => vpass);

  rena <= vpass and vena;

end arch_fpocore;
