--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.fpotyp_pkg.all;
use work.fpocfg_pkg.all;

package fpoinc_pkg is

  type capture_ctrl_rec is record
    act   : std_ulogic;
    mode  : trg_mod_typ;
    depth : ramadr_typ;
    drift : drift_typ;
  end record;

  type display_ctrl_rec is record
    scale : scale_typ;
    disp  : ramadr_typ;
  end record;

  type vgaout is record
    r, g, b : std_ulogic;
    h, v    : std_ulogic;
  end record;

  function trace_data_width(typ : trace_typ_typ) return natural;
  function fpo_item_typ(i : natural) return trace_typ_typ;
  function fpo_scan_lines(i : natural) return natural;
  function fpo_item_low(i : natural) return natural;
  function fpo_item_width(i : natural) return natural;
  function fpo_total_width return natural;

  subtype fpo_data_typ is std_ulogic_vector(fpo_total_width - 1 downto 0);
  subtype trace_range_typ is natural range fpo_config'range;

end fpoinc_pkg;

package body fpoinc_pkg is

  function trace_data_width(typ : trace_typ_typ) return natural is
  begin
    return trace_properties(typ).width;
  end;

  function fpo_scan_lines(i : natural) return natural is
  begin
    return trace_properties(fpo_config(i).trace_typ).lines;
  end;

  function fpo_item_width(i : natural) return natural is
  begin
    return trace_properties(fpo_config(i).trace_typ).width;
  end;

  function fpo_item_typ(i : natural) return trace_typ_typ is
  begin
    return fpo_config(i).trace_typ;
  end;

  function fpo_total_width return natural is
    variable l : natural := 0;
  begin
    for i in fpo_config'range loop
      l := l + fpo_item_width(i);
    end loop;
    return l;
  end;

  function fpo_item_range(i : natural) return std_ulogic_vector is
    variable r : fpo_data_typ;
    variable l : natural := 0;
  begin
    for j in fpo_config'range loop
      if j = i then
        exit;
      end if;
      l := l + fpo_item_width(j);
    end loop;
    return r(l + fpo_item_width(i) - 1 downto l);
  end;

  function fpo_item_low(i : natural) return natural is
    variable l : natural := 0;
  begin
    for j in fpo_config'range loop
      if j = i then
        exit;
      end if;
      l := l + fpo_item_width(j);
    end loop;
    return l;
  end;

end fpoinc_pkg;
