--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.videomode_pkg.all;
use work.fpotyp_pkg.all;
use work.tracedmux_pkg.all;

package main_pkg is

  component main is
    port(clk, ena : in std_ulogic; line : std_ulogic_vector(7 downto 0); selbus : selbus_typ; tracedbus : tracedbus_typ; scale : in scale_typ; col : in col_typ; color : out color_bus_typ);
  end component;

end main_pkg;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.videomode_pkg.all;
use work.fpotyp_pkg.all;
use work.tracedmux_pkg.all;
use work.traces_pkg.all;

entity main is
  port(clk, ena : in std_ulogic; line : std_ulogic_vector(7 downto 0); selbus : selbus_typ; tracedbus : tracedbus_typ; scale : in scale_typ; col : in col_typ; color : out color_bus_typ);
end main;

architecture arch_main of main is

  type tracecolor_typ is array (trace_typ_typ) of color_bus_typ;

  signal tracecolor : tracecolor_typ;

begin

  it_bit16 : trace_bit16 port map(
    clk       => clk,
    ena       => ena,
    trace_ena => selbus(tt_bit16),
    d         => tracedbus.bit16,
    scale     => scale,
    line      => line(3 downto 0),
    col       => col,
    color     => tracecolor(tt_bit16)
    );

  it_bus16w4 : trace_bus16w4 port map(
    clk       => clk,
    ena       => ena,
    trace_ena => selbus(tt_bus16w4),
    d         => tracedbus.bus16w4,
--      scale => scale,
    line      => line(3 downto 0),
    col       => col,
    color     => tracecolor(tt_bus16w4)
    );

  it_dot64u : trace_dot64u port map(
    clk       => clk,
    ena       => ena,
    trace_ena => selbus(tt_dot64u),
    d         => tracedbus.dot64u,
    scale     => scale,
    line      => line(5 downto 0),
    col       => col,
    color     => tracecolor(tt_dot64u)
    );

  it_bar64u : trace_bar64u port map(
    clk       => clk,
    ena       => ena,
    trace_ena => selbus(tt_bar64u),
    d         => tracedbus.bar64u,
    scale     => scale,
    line      => line(5 downto 0),
    col       => col,
    color     => tracecolor(tt_bar64u)
    );

  it_bar256u : trace_bar256u port map(
    clk       => clk,
    ena       => ena,
    trace_ena => selbus(tt_bar256u),
    d         => tracedbus.bar256u,
    scale     => scale,
    line      => line(7 downto 0),
    col       => col,
    color     => tracecolor(tt_bar256u)
    );

  it_separ4 : trace_separ4 port map(
    clk       => clk,
    ena       => ena,
    trace_ena => selbus(tt_separ4),
    line      => line(7 downto 0),
    col       => col,
    color     => tracecolor(tt_separ4)
    );

  process(tracecolor)
    variable c : color_bus_typ;
  begin
    c := ('0', '0', '0', '0');
    for tt in trace_typ_typ loop
      c.r   := c.r or tracecolor(tt).r;
      c.g   := c.g or tracecolor(tt).g;
      c.b   := c.b or tracecolor(tt).b;
      c.pix := c.pix or tracecolor(tt).pix;
    end loop;
    color.r   <= c.r;
    color.g   <= c.g;
    color.b   <= c.b;
    color.pix <= c.pix;
  end process;

end arch_main;
