--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.videomode_pkg.all;

package rastersync_pkg is

  component rastersync is
    port (clk, ena : in std_ulogic; hactive, vactive, next_line, hsync, vsync : out std_ulogic; vcol : out col_typ; vrow : out row_typ);
  end component;

end rastersync_pkg;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.axissync_pkg.all;
use work.videomode_pkg.all;

entity rastersync is
  port (clk, ena : in std_ulogic; hactive, vactive, next_line, hsync, vsync : out std_ulogic; vcol : out col_typ; vrow : out row_typ);
end rastersync;

architecture arch_rastersync of rastersync is

  signal e, hs, vs, hv, vv : std_ulogic;
  signal col               : col_typ;
  signal row               : row_typ;

begin

  syncn : axissync generic map(frontporch => videotiming.hfrontporch,
                               syncpulse  => videotiming.hsyncpulse,
                               backporch  => videotiming.hbackporch,
                               visible    => videotiming.hvisible)
    port map(clk   => clk,
             ena   => ena,
             s     => hs,
             v     => hv,
             e     => e,
             coord => col);

  syncv : axissync generic map(frontporch => videotiming.vfrontporch,
                               syncpulse  => videotiming.vsyncpulse,
                               backporch  => videotiming.vbackporch,
                               visible    => videotiming.vvisible)
    port map(clk   => clk,
             ena   => e,
             s     => vs,
             v     => vv,
             e     => open,
             coord => row);

  hactive <= hv;

  vactive <= vv;

  vcol <= col;

  vrow <= row;

  next_line <= e;

  hsync <= hs xor videotiming.hsyncneg;
  vsync <= vs xor videotiming.hsyncneg;

end arch_rastersync;
