--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.fpocfg_pkg.all;
use work.fpoinc_pkg.all;

package sweep_pkg is

  component sweep is
    port (clk, ena, trg : in std_ulogic; capture_ctrl : in capture_ctrl_rec; sw, fill, run : out std_ulogic; adr, trgadr : out ramadr_typ; trgdrift : out drift_typ);
  end component;

end sweep_pkg;

library ieee;
use ieee.std_logic_1164.all;
use work.fpocfg_pkg.all;
use work.fpoinc_pkg.all;

entity sweep is
  port (clk, ena, trg : in std_ulogic; capture_ctrl : in capture_ctrl_rec; sw, fill, run : out std_ulogic; adr, trgadr : out ramadr_typ; trgdrift : out drift_typ);
end sweep;

architecture arch_sweep of sweep is

  signal ramadr, tadr, acquired : ramadr_typ;

  signal count, pos, tdrift : drift_typ;

  signal filled, rdy, snap, srun, run_ff, got : boolean;

begin

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if ramadr_typ'high > ramadr then
          ramadr <= ramadr + 1;
        else
          ramadr <= 0;
        end if;
      end if;
    end if;
  end process;

  adr <= ramadr;

  filled <= (capture_ctrl.drift <= acquired);

  fill <= '1' when not filled else '0';

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if got then
          acquired <= 0;
        else
          if ramadr_typ'high > acquired then
            acquired <= acquired + 1;
          end if;
        end if;
      end if;
    end if;
  end process;

  rdy <= filled and ('1' = capture_ctrl.act);  --filled and not act is stp

  snap <= rdy and ('1' = trg) and not run_ff;  --mask trigger

  srun <= run_ff or snap;

  run <= '1' when srun else '0';

  process(clk)
  begin
    if rising_edge(clk) then
      if '1'= ena then
        if got then
          run_ff <= false;
        else
          if snap then
            run_ff <= true;
          end if;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1'= ena then

        if snap then
          count <= capture_ctrl.drift + 1;
        else
          if srun then
            count <= count + 1;
          end if;
        end if;
      end if;
    end if;
  end process;

  pos <= capture_ctrl.drift when snap else count;

  got <= ((capture_ctrl.depth) <= pos) and srun;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1'= ena then

        if snap then
          tadr   <= ramadr;
          tdrift <= capture_ctrl.drift;
        end if;

      end if;
    end if;
  end process;

  trgadr <= ramadr when (got and snap) else tadr;

  trgdrift <= capture_ctrl.drift when (got and snap) else tdrift;

  sw <= '1' when got else '0';

end arch_sweep;
