--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.fpotyp_pkg.all;
use work.fpocfg_pkg.all;
use work.fpoinc_pkg.all;

package tracedmux_pkg is

  type selbus_typ is array (trace_typ_typ) of std_ulogic;

  type tracedbus_typ is record
    bit16   : std_ulogic;  --_vector(trace_data_width(tt_bit16) downto 0);
    bus16w4 : std_ulogic_vector(trace_data_width(tt_bus16w4) - 1 downto 0);
    dot64u  : std_ulogic_vector(trace_data_width(tt_dot64u) - 1 downto 0);
    bar64u  : std_ulogic_vector(trace_data_width(tt_bar64u) - 1 downto 0);
    bar256u : std_ulogic_vector(trace_data_width(tt_bar256u) - 1 downto 0);
  end record;

  component tracedmux is
    port(i : in trace_range_typ; e : in boolean; d : in fpo_data_typ; selbus : out selbus_typ; tracedbus : out tracedbus_typ);
  end component;

end tracedmux_pkg;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.fpotyp_pkg.all;
use work.fpocfg_pkg.all;
use work.fpoinc_pkg.all;
use work.tracedmux_pkg.all;

entity tracedmux is
  port(i : in trace_range_typ; e : in boolean; d : in fpo_data_typ; selbus : out selbus_typ; tracedbus : out tracedbus_typ);
end tracedmux;

architecture arch_tracedmux of tracedmux is

  function vor(x : std_ulogic_vector) return std_ulogic is
    variable y : std_ulogic_vector(x'length - 1 downto 0) := x;
    variable l : natural                                  := x'length;
  begin
    while l > 1 loop
      if l mod 2 > 0 then
        l                 := l / 2 + 1;
        y(l - 1 downto 0) := ('0' & y(2 * l - 2 downto l)) or y(l - 1 downto 0);
      else
        l                 := l / 2;
        y(l - 1 downto 0) := y(2 * l - 1 downto l) or y(l - 1 downto 0);
      end if;
    end loop;
    return y(0);
  end;

  function trace_muxb(typ : trace_typ_typ; i : trace_range_typ; d : fpo_data_typ) return std_ulogic is
    variable b : std_ulogic := '0';
  begin
    for j in fpo_config'range loop
      if (i = j) then
        if (typ = fpo_item_typ(j)) then
          b := d(fpo_item_low(j));
        end if;
      end if;
    end loop;
    return b;
  end;

  function trace_muxv(typ : trace_typ_typ; i : trace_range_typ; d : fpo_data_typ) return std_ulogic_vector is
    variable v : std_ulogic_vector(trace_data_width(typ) - 1 downto 0) := (others => '0');
  begin
    for j in fpo_config'range loop
      if (i = j) then
        if (typ = fpo_item_typ(j)) then
          if 0 < fpo_item_width(j) then
            v := d(fpo_item_low(j) + fpo_item_width(j) - 1 downto fpo_item_low(j));
          end if;
        end if;
      end if;
    end loop;
    return v;
  end;

  function trace_sel(typ : trace_typ_typ; i : trace_range_typ; e : boolean) return std_ulogic is
    variable sel : std_ulogic_vector(trace_range_typ) := (others => '0');
  begin
    for j in trace_range_typ loop
      if (i = j) and e then
        if (typ = fpo_item_typ(j)) then
          sel(j) := '1';
        end if;
      end if;
    end loop;
    return vor(sel);
  end;

begin

  tracedbus.bit16 <= trace_muxb(tt_bit16, i, d);

  tracedbus.bus16w4 <= trace_muxv(tt_bus16w4, i, d);

  tracedbus.dot64u <= trace_muxv(tt_dot64u, i, d);

  tracedbus.bar64u <= trace_muxv(tt_bar64u, i, d);

  tracedbus.bar256u <= trace_muxv(tt_bar256u, i, d);

  selgen : for tt in trace_typ_typ generate
    selbus(tt) <= trace_sel(tt, i, e);
  end generate;

end arch_tracedmux;
