--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.fpotyp_pkg.all;
use work.videomode_pkg.all;

package traces_pkg is

  component trace_bit16
    port (clk, ena, trace_ena, d : in std_ulogic; scale : in scale_typ; line : in std_ulogic_vector(3 downto 0); col : in col_typ; color : out color_bus_typ);
  end component;

  component trace_bus16w4 is
    port (clk, ena, trace_ena : in std_ulogic; d : in std_ulogic_vector(3 downto 0); line : in std_ulogic_vector(3 downto 0); col : in col_typ; color : out color_bus_typ);
  end component;

  component trace_dot64u is
    port (clk, ena, trace_ena : in std_ulogic; d : in std_ulogic_vector(5 downto 0); scale : in scale_typ; line : in std_ulogic_vector(5 downto 0); col : in col_typ; color : out color_bus_typ);
  end component;

  component trace_bar64u is
    port (clk, ena, trace_ena : in std_ulogic; d : in std_ulogic_vector(5 downto 0); scale : in scale_typ; line : in std_ulogic_vector(5 downto 0); col : in col_typ; color : out color_bus_typ);
  end component;

  component trace_bar256u is
    port (clk, ena, trace_ena : in std_ulogic; d : in std_ulogic_vector(7 downto 0); scale : in scale_typ; line : in std_ulogic_vector(7 downto 0); col : in col_typ; color : out color_bus_typ);
  end component;

  component trace_separ4 is
    port (clk, ena, trace_ena : in std_ulogic; line : in std_ulogic_vector(7 downto 0); col : in col_typ; color : out color_bus_typ);
  end component;

end traces_pkg;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.fpotyp_pkg.all;
use work.videomode_pkg.all;

entity trace_bit16 is
  port (clk, ena, trace_ena, d : in std_ulogic; scale : in scale_typ; line : in std_ulogic_vector(3 downto 0); col : in col_typ; color : out color_bus_typ);
end trace_bit16;

architecture arch_trace_bit16 of trace_bit16 is

  signal q, pix : std_ulogic;
  signal qd     : std_ulogic_vector(1 downto 0);

begin

  color.pix <= trace_ena;
  color.r   <= '0';
  color.g   <= pix and trace_ena;
  color.b   <= '0';

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
--        if "11" = col(1 downto 0) then  --scale4
        if ("11" = col(1 downto 0)) or (std_ulogic_vector(to_unsigned(0, col_typ'length)) = col) then
          q <= d;
        end if;
      end if;
    end if;
  end process;

  qd <= q & d;

  process (col, line, scale, d, qd)
  begin

    case scale is
      when 2 => --scale4

        case qd is
          when "01" =>
            if line(3) = '1' then
              case line(2 downto 0) is
                when "110" | "111" => pix <= '0';
                when "101"         => pix <= (not col(1)) and (not col(0));
                when others        => pix <= (not col(1)) and col(0);
              end case;
            else
              case line(2 downto 0) is
                when "000" | "001" => pix <= '0';
                when "010"         => pix <= col(1) and col(0);
                when others        => pix <= col(1) and not col(0);
              end case;
            end if;

          when "10" =>
            if line(3) = '1' then
              case line(2 downto 0) is
                when "110" | "111" => pix <= '0';
                when "101"         => pix <= col(1) and col(0);
                when others        => pix <= col(1) and not col(0);
              end case;
            else
              case line(2 downto 0) is
                when "000" | "001" => pix <= '0';
                when "010"         => pix <= (not col(1)) and (not col(0));
                when others        => pix <= (not col(1)) and col(0);
              end case;
            end if;
          when others =>

            case line(3 downto 0) is
              when "0010" => pix <= d;
              when "1101" => pix <= not d;
              when others => pix <= '0';
            end case;

        end case;

--    when 3 => --scale8
-----------------
--11000000
--00100000
--00100000
--00010000
--00010000
--00010000
--00001000
--00001000
--00001000
--00000100
--00000100
--00000011

      when others => --scale1 & scale2

        case line(3 downto 0) is
          when "0010" => pix <= d;
          when "1101" => pix <= not d;
          when others => pix <= '0';
        end case;

    end case;

  end process;

end arch_trace_bit16;


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.fpotyp_pkg.all;
use work.videomode_pkg.all;

entity trace_bus16w4 is
  port (clk, ena, trace_ena : in std_ulogic; d : in std_ulogic_vector(3 downto 0); line : in std_ulogic_vector(3 downto 0); col : in col_typ; color : out color_bus_typ);
end trace_bus16w4;

architecture arch_trace_bus16w4 of trace_bus16w4 is

  signal pix : std_ulogic;

begin

  color.pix <= trace_ena;
  color.r   <= '0';
  color.g   <= pix and trace_ena;
  color.b   <= (not pix) and (line(1) and line(0)) and (not (line(0) and line(1) and line(2) and line(3))) and trace_ena;

  pix <= d(to_integer(unsigned(line(3 downto 2)))) and not line(1);

end arch_trace_bus16w4;

library ieee;
use ieee.std_logic_1164.all;
use work.fpotyp_pkg.all;
use work.videomode_pkg.all;

entity trace_dot64u is
  port (clk, ena, trace_ena : in std_ulogic; d : in std_ulogic_vector(5 downto 0); scale : in scale_typ; line : in std_ulogic_vector(5 downto 0); col : in col_typ; color : out color_bus_typ);
end trace_dot64u;

architecture arch_trace_dot64u of trace_dot64u is

  signal pix : std_ulogic;

begin

  color.pix <= trace_ena;
  color.r   <= '0';
  color.g   <= pix and trace_ena;
  color.b   <= '0';

  pix <= '1' when (not line) = d else '0';

end arch_trace_dot64u;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.fpotyp_pkg.all;
use work.videomode_pkg.all;

entity trace_bar64u is
  port (clk, ena, trace_ena : in std_ulogic; d : in std_ulogic_vector(5 downto 0); scale : in scale_typ; line : in std_ulogic_vector(5 downto 0); col : in col_typ; color : out color_bus_typ);
end trace_bar64u;

architecture arch_trace_bar64u of trace_bar64u is

  signal pix : std_ulogic;

begin

  color.pix <= trace_ena;
  color.r   <= '0';
  color.g   <= pix and trace_ena;
  color.b   <= '0';

  pix <= '1' when unsigned(not line) <= unsigned(d) else '0';

end arch_trace_bar64u;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.fpotyp_pkg.all;
use work.videomode_pkg.all;

entity trace_bar256u is
  port (clk, ena, trace_ena : in std_ulogic; d : in std_ulogic_vector(7 downto 0); scale : in scale_typ; line : in std_ulogic_vector(7 downto 0); col : in col_typ; color : out color_bus_typ);
end trace_bar256u;

architecture arch_trace_bar256u of trace_bar256u is

  signal pix : std_ulogic;

begin

  color.pix <= trace_ena;
  color.r   <= '0';
  color.g   <= pix and trace_ena;
  color.b   <= '0';

  pix <= '1' when unsigned(not line) <= unsigned(d) else '0';

end arch_trace_bar256u;

library ieee;
use ieee.std_logic_1164.all;
use work.videomode_pkg.all;

entity trace_separ4 is
  port (clk, ena, trace_ena : in std_ulogic; line : in std_ulogic_vector(7 downto 0); col : in col_typ; color : out color_bus_typ);
end trace_separ4;

architecture arch_trace_separ4 of trace_separ4 is

begin

  color.pix <= trace_ena;
  color.r   <= trace_ena;
  color.g   <= '0';
  color.b   <= '0';

end arch_trace_separ4;
