--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.videomode_pkg.all;
use work.fpoinc_pkg.all;

package tracescan_pkg is

  component tracescan is
    port(clk, ena, vactive, next_line : in std_ulogic; line : out std_ulogic_vector(7 downto 0); trace : out trace_range_typ; valid : out boolean);
  end component;

end tracescan_pkg;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.fpocfg_pkg.all;
use work.fpoinc_pkg.all;

entity tracescan is
  port(clk, ena, vactive, next_line : in std_ulogic; line : out std_ulogic_vector(7 downto 0); trace : out trace_range_typ; valid : out boolean);
end tracescan;

architecture arch_tracescan of tracescan is

  signal line_count                       : natural range 0 to 255;
  signal trace_count                      : trace_range_typ;
  signal line_last, trace_last, sel_valid : boolean;

begin

  line_last <= (fpo_scan_lines(trace_count) - 1) = line_count;

  trace_last <= fpo_config'high = trace_count;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = vactive then
          if next_line = '1' then
            if line_last then
              if trace_last then
                sel_valid <= false;
              else
                trace_count <= trace_count + 1;
                line_count  <= 0;
              end if;
            else
              line_count <= line_count + 1;
            end if;
          end if;
        else
          trace_count <= 0;
          line_count  <= 0;
          sel_valid   <= true;
        end if;
      end if;
    end if;
  end process;

  line <= std_ulogic_vector(to_unsigned(line_count, 8));

  trace <= trace_count;

  valid <= sel_valid;

end arch_tracescan;
