--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.fpotyp_pkg.all;

package trigger_pkg is

  component trigger is
    port (clk, ena, act, d : in std_ulogic; trg_mod : in trg_mod_typ; holdoff : in natural; trg, ready : out std_ulogic);
  end component;

end trigger_pkg;

library ieee;
use ieee.std_logic_1164.all;
use work.fpotyp_pkg.all;

entity trigger is
  port (clk, ena, act, d : in std_ulogic; trg_mod : in trg_mod_typ; holdoff : in natural; trg, ready : out std_ulogic);
end trigger;

architecture arch_trigger of trigger is

  constant filter : boolean := false;  -- when d is asynchronous to clk use filter to eliminate trg signal bounce and skew (and reduce metastability likelihood)

  signal t0, t1, rdy, rdys, r0 : std_ulogic;

begin

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        t0 <= d;
        t1 <= t0;
      end if;
    end if;
  end process;

  rdy <= act when norm = trg_mod else rdys;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        r0 <= act;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then

        if '1' = (t0 and not t1) then
          rdys <= '0';
        else
          if '1' = (act and not r0) then
            rdys <= '1';
          end if;
        end if;

      end if;
    end if;
  end process;

  fl : if filter generate
    trg <= (t0 and not t1) and rdy;
  end generate;

  nfl : if not filter generate
    trg <= (d and not t0) and rdy;
  end generate;

  ready <= rdy;

end arch_trigger;
