--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
package trinity_pkg is

  component trinity is
    port (clk, ena, step, hold : in std_ulogic; wsel, vsel : out std_ulogic_vector(2 downto 0));
  end component;

end trinity_pkg;

library ieee;
use ieee.std_logic_1164.all;

entity trinity is
  port (clk, ena, step, hold : in std_ulogic; wsel, vsel : out std_ulogic_vector(2 downto 0));
end trinity;

architecture arch_trinity of trinity is

  type tcnt is range 0 to 2;

  function dc3(x : tcnt) return std_ulogic_vector is
    variable t : std_ulogic_vector(2 downto 0);
  begin
    case x is
      when 0      => t := "001";
      when 1      => t := "010";
      when others => t := "100";
    end case;
    return t;
  end;

  function cmp3m(x, y : tcnt) return boolean is
    variable t : tcnt;
  begin
    t := (x - y) mod 3;
    return not (1 = t);
  end;

  function cmp3(x, y : tcnt) return boolean is
  begin
    return ((2 = x) and (0 = y)) or ((1 = x) and (2 = y)) or ((0 = x) and (1 = y));
  end;

  signal wrt, rec, img : tcnt;

begin

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = step then
          if cmp3(img, wrt) then
            if 2 <= wrt then
              wrt <= 0;
            else
              wrt <= wrt + 1;
            end if;
          else
            if 0 = wrt then
              wrt <= 2;
            else
              wrt <= wrt - 1;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = step then
          rec <= wrt;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '0' = (hold or step) then
          img <= rec;
        end if;
      end if;
    end if;
  end process;

  wsel <= dc3(wrt);
  vsel <= dc3(img);

end arch_trinity;
