--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.fpocfg_pkg.all;
use work.fpoinc_pkg.all;

package triram_pkg is

  component triram is
    port (clk, ena, rclk, rena : in std_ulogic; wsel, vsel : in std_ulogic_vector(2 downto 0); wadr, radr : ramadr_typ; d : in fpo_data_typ; q : out fpo_data_typ);
  end component;

end triram_pkg;

library ieee;
use ieee.std_logic_1164.all;
use work.fpocfg_pkg.all;
use work.fpoinc_pkg.all;

entity triram is
  port (clk, ena, rclk, rena : in std_ulogic; wsel, vsel : in std_ulogic_vector(2 downto 0); wadr, radr : ramadr_typ; d : in fpo_data_typ; q : out fpo_data_typ);
end triram;

architecture arch_triram of triram is

  type ramblock_typ is array (ramadr_typ) of fpo_data_typ;

  function fil(x : std_ulogic; n : positive) return std_ulogic_vector is
    variable t : std_ulogic_vector(n - 1 downto 0);
  begin
    t := (others => x);
    return t;
  end;

  signal rama, ramb, ramc : ramblock_typ;
  signal qa, qb, qc       : fpo_data_typ;

begin

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = wsel(0) then
          rama(wadr) <= d;
        end if;
        if '1' = wsel(1) then
          ramb(wadr) <= d;
        end if;
        if '1' = wsel(2) then
          ramc(wadr) <= d;
        end if;
      end if;
    end if;
  end process;

  process(rclk)
  begin
    if rising_edge(rclk) then
      if '1' = rena then
        if '1' = vsel(0) then
          qa <= rama(radr);
        end if;
        if '1' = vsel(1) then
          qb <= ramb(radr);
        end if;
        if '1' = vsel(2) then
          qc <= ramc(radr);
        end if;
      end if;
    end if;
  end process;

  q <= (qa and fil(vsel(0), fpo_total_width)) or (qb and fil(vsel(1), fpo_total_width)) or (qc and fil(vsel(2), fpo_total_width));

end arch_triram;
