--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.videomode_pkg.all;
use work.fpocfg_pkg.all;
use work.fpoinc_pkg.all;

package videoadr_pkg is

  component videoadr is
    port (clk, ena, pass, hactive, vactive : in std_ulogic; capture_ctrl : in capture_ctrl_rec; display_ctrl : in display_ctrl_rec; badr : in ramadr_typ; blank : out std_ulogic; adr, count : out ramadr_typ);
  end component;

end videoadr_pkg;


library ieee;
use ieee.std_logic_1164.all;
use work.videomode_pkg.all;
use work.fpocfg_pkg.all;
use work.fpoinc_pkg.all;

entity videoadr is
  port (clk, ena, pass, hactive, vactive : in std_ulogic; capture_ctrl : in capture_ctrl_rec; display_ctrl : in display_ctrl_rec; badr : in ramadr_typ; blank : out std_ulogic; adr, count : out ramadr_typ);
end videoadr;

architecture arch_videoadr of videoadr is

  signal gl_r, gl_blank : std_ulogic;
  signal cadr, gl_count : ramadr_typ;

begin

  gl_r <= not hactive;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = gl_r then
--        cadr <= (badr + display_ctrl.disp +  1) rem ramlength;
          cadr <= (badr + display_ctrl.disp + 1) mod ramlength;
        else
          if '1' = pass then
            if ramadr_typ'high > cadr then
              cadr <= cadr + 1;
            else
              cadr <= 0;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;

--  adr <= (badr + display_ctrl.disp) rem ramlength when '1' = gl_r else cadr;
  adr <= (badr + display_ctrl.disp) mod ramlength when '1' = gl_r else cadr;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = gl_r then
          gl_count <= (display_ctrl.disp + 1);
        else
          if '1' = pass then
            if capture_ctrl.depth > gl_count then
              gl_count <= gl_count + 1;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if '1' = gl_r then
          gl_blank <= '0';
          blank    <= '0';
        else
          if '1' = pass then
            if capture_ctrl.depth <= gl_count then
              gl_blank <= '1';
              blank    <= gl_blank;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;

  count <= display_ctrl.disp when '1' = gl_r else gl_count;

end arch_videoadr;
