--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;

package videomode_pkg is

  function wh(x : positive) return natural;

  type vgatiming is record
    freq : positive;

    hfrontporch : positive;
    hsyncpulse  : positive;
    hbackporch  : positive;
    hvisible    : positive;

    vfrontporch : positive;
    vsyncpulse  : positive;
    vbackporch  : positive;
    vvisible    : positive;

    hsyncneg : std_ulogic;
    vsyncneg : std_ulogic;

  end record;

  function videotiming return vgatiming;

  subtype col_typ is std_ulogic_vector(wh(videotiming.hvisible) downto 0);
  subtype row_typ is std_ulogic_vector(wh(videotiming.vvisible) downto 0);

  type color_bus_typ is record
    pix     : std_ulogic;
    r, g, b : std_ulogic;
  end record;

end videomode_pkg;

use work.fpotyp_pkg.all;
use work.fpocfg_pkg.all;

package body videomode_pkg is

  function videotiming return vgatiming is
    variable vt : vgatiming;
  begin
    case vgamode is
      when e_800x600_72 =>              --  800x600 72Hz
        vt.freq := 50000000;

        vt.hfrontporch := 56;
        vt.hsyncpulse  := 120;
        vt.hbackporch  := 64;
        vt.hvisible    := 800;

        vt.vfrontporch := 37;
        vt.vsyncpulse  := 6;
        vt.vbackporch  := 23;
        vt.vvisible    := 600;

        vt.hsyncneg := '0';
        vt.vsyncneg := '0';
      when e_1280x1024_75 =>            -- 1280x1024 75Hz
        vt.freq := 135000000;

        vt.hfrontporch := 16;
        vt.hsyncpulse  := 144;
        vt.hbackporch  := 248;
        vt.hvisible    := 1280;

        vt.vfrontporch := 1;
        vt.vsyncpulse  := 3;
        vt.vbackporch  := 38;
        vt.vvisible    := 1024;

        vt.hsyncneg := '0';
        vt.vsyncneg := '0';
    end case;
    return vt;
  end;

  function wh(x : positive) return natural is
    variable l : natural  := 0;
    variable t : positive := x;
  begin
    while 1 < t loop
      l := l + 1;
      t := t / 2;
    end loop;
    return l;
  end;

end videomode_pkg;
