--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;
use work.videomode_pkg.all;

package sprite_pkg is

  component sprite is
    port (clk, ena : in std_ulogic; pix : out std_ulogic; x, col : in col_typ; y, row : in row_typ);
  end component;

end sprite_pkg;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.videomode_pkg.all;

entity sprite is
    port (clk, ena : in std_ulogic; pix : out std_ulogic; x, col : in col_typ; y, row : in row_typ);
end sprite;

architecture arch_sprite of sprite is

  constant xsize : positive := 16;
  constant ysize : positive := 16;

  constant x0 : natural := 8;
  constant y0 : natural := 8;

  signal sx : signed(wh(videotiming.hvisible) + 1 downto 0);
  signal sy : signed(wh(videotiming.vvisible) + 1 downto 0);

  type sprite_mask_typ is array (0 to ysize - 1, 0 to xsize - 1) of std_ulogic;

  constant sprite_mask : sprite_mask_typ := (
    (B"0000000000000000"),
    (B"0000000010000000"),
    (B"0000000111000000"),
    (B"0000001010100000"),
    (B"0000010010010000"),
    (B"0000100010001000"),
    (B"0001000010000100"),
    (B"0010000010000010"),
    (B"0111111111111111"),
    (B"0010000010000010"),
    (B"0001000010000100"),
    (B"0000100010001000"),
    (B"0000010010010000"),
    (B"0000001010100000"),
    (B"0000000111000000"),
    (B"0000000010000000"));

begin

  sx <= signed('0' & col) - (('0' & signed(x)) - x0);
  sy <= signed('0' & row) - (('0' & signed(y)) - y0);

  pix <= sprite_mask(to_integer(sy), to_integer(sx)) when ((0 <= sx) and (xsize > sx) and (0 <= sy) and (ysize > sy)) else '0';

end arch_sprite;
