--Copyright (C) 2015 Zen Lu and released under the GPLv3

library ieee;
use ieee.std_logic_1164.all;

package ps2host_pkg is

--reset by '1'
--ps2 host receiver without clock

  component ps2hrnc is
    port (reset, ps2c, ps2d : in std_ulogic; d : out std_ulogic_vector(7 downto 0); v : out std_ulogic);
  end component;

-- Apply 100..150 kHz to clk/ena (up to 200kHz should be fine)

  component ps2_htx is
    port (clk, ena, reset, strobe : in std_ulogic; ps2c, ps2d : inout std_ulogic; d : in std_ulogic_vector(7 downto 0); ack, busy : out std_ulogic);
  end component;

end ps2host_pkg;

library ieee;
use ieee.std_logic_1164.all;

entity ps2hrnc is
  port (reset, ps2c, ps2d : in std_ulogic; d : out std_ulogic_vector(7 downto 0); v : out std_ulogic);
end ps2hrnc;

architecture arch_ps2hrnc of ps2hrnc is

  type ps2_state_typ is range 0 to 10;  --(ready, data0, data1, data2, data3, data4, data5, data6, data7, parity, stop);

  constant ready  : ps2_state_typ := 0;
  constant data0  : ps2_state_typ := 1;
  constant data7  : ps2_state_typ := 8;
  constant parity : ps2_state_typ := 9;
  constant stop   : ps2_state_typ := 10;

  signal ps2_state       : ps2_state_typ;
  signal valid, data_xor : std_ulogic;
  signal data_reg        : std_ulogic_vector(7 downto 0);
  signal parity_ok       : boolean;

begin

  process(ps2c, reset)
  begin
    if '1' = reset then
      valid     <= '0';
      ps2_state <= ready;
      parity_ok <= false;
    else
      if falling_edge(ps2c) then
        parity_ok <= (parity = ps2_state) and (not data_xor = ps2d);
        if (stop = ps2_state) and parity_ok and ('1' = ps2d) then
          valid <= '1';
        else
          if (ready = ps2_state) and ('0' = ps2d) then
            valid <= '0';
          end if;
        end if;
        if (((ready = ps2_state) or (stop < ps2_state)) and ('0' = ps2d)) then
          ps2_state <= data0;
        else
          if ((data0 <= ps2_state) and (data7 >= ps2_state)) or ((parity = ps2_state) and (not data_xor = ps2d)) then  -- or ((ready = ps2_state) and ('0' = ps2d))
            ps2_state <= ps2_state + 1;
          else
            ps2_state <= ready;
          end if;
        end if;
        if (data0 <= ps2_state) and (data7 >= ps2_state) then
          for i in 0 to 6 loop
            data_reg(i) <= data_reg(i + 1);
          end loop;
          data_reg(7) <= ps2d;
        end if;
      end if;
    end if;
  end process;

  data_xor <= ((data_reg(0) xor data_reg(1)) xor (data_reg(2) xor data_reg(3))) xor ((data_reg(4) xor data_reg(5)) xor (data_reg(6) xor data_reg(7)));

  d <= data_reg;

  v <= valid;

end arch_ps2hrnc;


library ieee;
use ieee.std_logic_1164.all;


entity ps2_htx is
  port (clk, ena, reset, strobe : in std_ulogic; ps2c, ps2d : inout std_ulogic; d : in std_ulogic_vector(7 downto 0); ack, busy : out std_ulogic);
end ps2_htx;

architecture arch_ps2_htx of ps2_htx is

  function od(x : std_ulogic) return std_ulogic is
    variable temp : std_ulogic := 'Z';
  begin
    if '0' = x then
      temp := '0';
    end if;
    return temp;
  end;

  constant ultimate : natural := 54;

  signal last, step, load1  : boolean;
  signal state              : natural range 0 to ultimate;
  signal data_reg           : std_ulogic_vector(7 downto 0);
  signal parity_ff, ps2c_ff : std_ulogic;

begin

  last <= ((ultimate = state) and step) or (ultimate < state);

  process(clk, reset)
  begin
    if '1' = reset then
      state <= 0;
    else
      if rising_edge(clk) then
        if '1' = ena then
          if last then
            state <= 0;
          else
            if load1 then
              state <= 1;
            else
              if step then
                state <= state + 1;
              end if;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        ps2c_ff <= ps2c;
      end if;
    end if;
  end process;

  process(clk)
  begin
    if rising_edge(clk) then
      if '1' = ena then
        if ('1' = strobe) and (0 = state) then
          data_reg  <= d;
          parity_ff <= '1';
        else
          if step then
            case state is
              when 35 | 37 | 39 | 41 | 43 | 45 | 47 | 49 =>
                parity_ff <= parity_ff xor data_reg(0);
                for i in 0 to 6 loop
                  data_reg(i) <= data_reg(i + 1);
                end loop;
              when others =>
                null;
            end case;
          end if;
        end if;
      end if;
    end if;
  end process;

  load1 <= (('1' /= (ps2c and ps2d)) and (1 <= state) and (15 >= state));

  process(state, strobe, ps2d, ps2c, ps2c_ff)
  begin
    case state is
      when 0 =>
        step <= '1' = strobe;
      when 1 to 31 =>
        step <= true;
      when 32 | 34 | 36 | 38 | 40 | 42 | 44 | 46 | 48 | 50 | 52 =>
        step <= '1' = (ps2c and ps2c_ff);
      when 33 | 35 | 37 | 39 | 41 | 43 | 45 | 47 | 49 | 51 | 53 =>
        step <= '0' = (ps2c or ps2c_ff);
      when 54 =>
        step <= true;
      when others =>
        step <= true;
        assert false;
    end case;
  end process;

  ps2c <= '0' when (16 <= state) and (31 >= state) else 'Z';

  process(state, data_reg, parity_ff)
  begin
    case state is
      when 0 to 30 =>
        ps2d <= 'Z';
      when 31 to 33 =>
        ps2d <= '0';
      when 34 to 49 =>
        ps2d <= od(data_reg(0));
      when 50 | 51 =>
        ps2d <= od(parity_ff);
      when others =>
        ps2d <= 'Z';
    end case;
  end process;

  busy <= '1' when (1 <= state) else '0';

  process(clk, reset)
  begin
    if '1' = reset then
      ack <= '0';
    else
      if rising_edge(clk) then
        if '1' = ena then
          if ('1' = strobe) and (0 = state) then
            ack <= '0';
          else
            if (53 = state) and ('0' = (ps2c or ps2c_ff)) then
              ack <= '1';
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;

end arch_ps2_htx;
